@@ -19,10 +19,11 @@
 
 package org.apache.guacamole.auth.jdbc.tunnel;
 
-import com.google.inject.Inject;
 import java.util.Date;
 import java.util.UUID;
+import org.apache.guacamole.auth.jdbc.connection.ConnectionRecordModel;
 import org.apache.guacamole.auth.jdbc.connection.ModeledConnection;
+import org.apache.guacamole.auth.jdbc.connection.ModeledConnectionRecord;
 import org.apache.guacamole.auth.jdbc.connectiongroup.ModeledConnectionGroup;
 import org.apache.guacamole.auth.jdbc.sharing.SharedConnectionMap;
 import org.apache.guacamole.auth.jdbc.sharing.SharedObjectManager;
@@ -31,7 +32,6 @@ import org.apache.guacamole.auth.jdbc.user.RemoteAuthenticatedUser;
 import org.apache.guacamole.net.AbstractGuacamoleTunnel;
 import org.apache.guacamole.net.GuacamoleSocket;
 import org.apache.guacamole.net.GuacamoleTunnel;
-import org.apache.guacamole.net.auth.ConnectionRecord;
 
 
 /**
@@ -39,41 +39,31 @@ import org.apache.guacamole.net.auth.ConnectionRecord;
  * the associated connection has not yet ended, getEndDate() will always return
  * null. The associated start date will be the time of this objects creation.
  */
-public class ActiveConnectionRecord implements ConnectionRecord {
+public class ActiveConnectionRecord extends ModeledConnectionRecord {
 
     /**
      * The user that connected to the connection associated with this connection
      * record.
      */
-    private RemoteAuthenticatedUser user;
+    private final RemoteAuthenticatedUser user;
 
     /**
      * The balancing group from which the associated connection was chosen, if
      * any. If no balancing group was used, this will be null.
      */
-    private ModeledConnectionGroup balancingGroup;
+    private final ModeledConnectionGroup balancingGroup;
 
     /**
      * The connection associated with this connection record.
      */
-    private ModeledConnection connection;
+    private final ModeledConnection connection;
 
     /**
      * The sharing profile that was used to access the connection associated
      * with this connection record. If the connection was accessed directly
      * (without involving a sharing profile), this will be null.
      */
-    private ModeledSharingProfile sharingProfile;
-
-    /**
-     * The time this connection record was created.
-     */
-    private final Date startDate = new Date();
-
-    /**
-     * The UUID that will be assigned to the underlying tunnel.
-     */
-    private final UUID uuid = UUID.randomUUID();
+    private final ModeledSharingProfile sharingProfile;
 
     /**
      * The connection ID of the connection as determined by guacd, not to be
@@ -91,8 +81,7 @@ public class ActiveConnectionRecord implements ConnectionRecord {
     /**
      * Map of all currently-shared connections.
      */
-    @Inject
-    private SharedConnectionMap connectionMap;
+    private final SharedConnectionMap connectionMap;
 
     /**
      * Manager which tracks all share keys associated with this connection
@@ -111,7 +100,57 @@ public class ActiveConnectionRecord implements ConnectionRecord {
     };
 
     /**
-     * Initializes this connection record, associating it with the given user,
+     * Creates a new connection record model object, associating it with the
+     * given user, connection, and sharing profile. The given sharing profile
+     * MUST be the sharing profile that was used to share access to the given
+     * connection. The start date of this connection record will be the time of
+     * its creation. No end date will be assigned.
+     *
+     * @param user
+     *     The user that connected to the connection associated with this
+     *     connection record.
+     *
+     * @param connection
+     *     The connection to associate with this connection record.
+     *
+     * @param sharingProfile
+     *     The sharing profile that was used to share access to the given
+     *     connection, or null if no sharing profile was used.
+     *
+     * @return
+     *     A new connection record model object associated with the given user,
+     *     connection, and sharing profile, and having the current date/time as
+     *     its start date.
+     */
+    private static ConnectionRecordModel createModel(RemoteAuthenticatedUser user,
+            ModeledConnection connection,
+            ModeledSharingProfile sharingProfile) {
+
+        // Create model object representing an active connection that started
+        // at the current time ...
+        ConnectionRecordModel recordModel = new ConnectionRecordModel();
+        recordModel.setStartDate(new Date());
+
+        // ... was established by the given user ...
+        recordModel.setUsername(user.getIdentifier());
+        recordModel.setRemoteHost(user.getRemoteHost());
+
+        // ... to the given connection ...
+        recordModel.setConnectionIdentifier(connection.getIdentifier());
+        recordModel.setConnectionName(connection.getName());
+
+        // ... using the given sharing profile (if any)
+        if (sharingProfile != null) {
+            recordModel.setSharingProfileIdentifier(sharingProfile.getIdentifier());
+            recordModel.setSharingProfileName(sharingProfile.getName());
+        }
+
+        return recordModel;
+        
+    }
+
+    /**
+     * Creates a new ActiveConnectionRecord associated with the given user,
      * connection, balancing connection group, and sharing profile. The given
      * balancing connection group MUST be the connection group from which the
      * given connection was chosen, and the given sharing profile MUST be the
@@ -119,6 +158,10 @@ public class ActiveConnectionRecord implements ConnectionRecord {
      * The start date of this connection record will be the time of its
      * creation.
      *
+     * @param connectionMap
+     *     The SharedConnectionMap instance tracking all active shared
+     *     connections.
+     *
      * @param user
      *     The user that connected to the connection associated with this
      *     connection record.
@@ -134,10 +177,13 @@ public class ActiveConnectionRecord implements ConnectionRecord {
      *     The sharing profile that was used to share access to the given
      *     connection, or null if no sharing profile was used.
      */
-    private void init(RemoteAuthenticatedUser user,
+    public ActiveConnectionRecord(SharedConnectionMap connectionMap,
+            RemoteAuthenticatedUser user,
             ModeledConnectionGroup balancingGroup,
             ModeledConnection connection,
             ModeledSharingProfile sharingProfile) {
+        super(createModel(user, connection, sharingProfile));
+        this.connectionMap = connectionMap;
         this.user = user;
         this.balancingGroup = balancingGroup;
         this.connection = connection;
@@ -145,12 +191,16 @@ public class ActiveConnectionRecord implements ConnectionRecord {
     }
    
     /**
-     * Initializes this connection record, associating it with the given user,
+     * Creates a new ActiveConnectionRecord associated with the given user,
      * connection, and balancing connection group. The given balancing
      * connection group MUST be the connection group from which the given
      * connection was chosen. The start date of this connection record will be
      * the time of its creation.
      *
+     * @param connectionMap
+     *     The SharedConnectionMap instance tracking all active shared
+     *     connections.
+     *
      * @param user
      *     The user that connected to the connection associated with this
      *     connection record.
@@ -161,17 +211,22 @@ public class ActiveConnectionRecord implements ConnectionRecord {
      * @param connection
      *     The connection to associate with this connection record.
      */
-    public void init(RemoteAuthenticatedUser user,
+    public ActiveConnectionRecord(SharedConnectionMap connectionMap,
+            RemoteAuthenticatedUser user,
             ModeledConnectionGroup balancingGroup,
             ModeledConnection connection) {
-        init(user, balancingGroup, connection, null);
+        this(connectionMap, user, balancingGroup, connection, null);
     }
 
     /**
-     * Initializes this connection record, associating it with the given user
+     * Creates a new ActiveConnectionRecord associated with the given user,
      * and connection. The start date of this connection record will be the time
      * of its creation.
      *
+     * @param connectionMap
+     *     The SharedConnectionMap instance tracking all active shared
+     *     connections.
+     *
      * @param user
      *     The user that connected to the connection associated with this
      *     connection record.
@@ -179,18 +234,22 @@ public class ActiveConnectionRecord implements ConnectionRecord {
      * @param connection
      *     The connection to associate with this connection record.
      */
-    public void init(RemoteAuthenticatedUser user,
-            ModeledConnection connection) {
-        init(user, null, connection);
+    public ActiveConnectionRecord(SharedConnectionMap connectionMap,
+            RemoteAuthenticatedUser user, ModeledConnection connection) {
+        this(connectionMap, user, null, connection);
     }
 
     /**
-     * Initializes this connection record, associating it with the given user,
+     * Creates a new ActiveConnectionRecord associated with the given user,
      * active connection, and sharing profile. The given sharing profile MUST be
      * the sharing profile that was used to share access to the given
      * connection. The start date of this connection record will be the time of
      * its creation.
      *
+     * @param connectionMap
+     *     The SharedConnectionMap instance tracking all active shared
+     *     connections.
+     *
      * @param user
      *     The user that connected to the connection associated with this
      *     connection record.
@@ -204,10 +263,11 @@ public class ActiveConnectionRecord implements ConnectionRecord {
      *     connection, or null if no sharing profile should be used (access to
      *     the connection is unrestricted).
      */
-    public void init(RemoteAuthenticatedUser user,
+    public ActiveConnectionRecord(SharedConnectionMap connectionMap,
+            RemoteAuthenticatedUser user,
             ActiveConnectionRecord activeConnection,
             ModeledSharingProfile sharingProfile) {
-        init(user, null, activeConnection.getConnection(), sharingProfile);
+        this(connectionMap, user, null, activeConnection.getConnection(), sharingProfile);
         this.connectionID = activeConnection.getConnectionID();
     }
 
@@ -286,63 +346,6 @@ public class ActiveConnectionRecord implements ConnectionRecord {
         return sharingProfile == null;
     }
 
-    @Override
-    public String getConnectionIdentifier() {
-        return connection.getIdentifier();
-    }
-
-    @Override
-    public String getConnectionName() {
-        return connection.getName();
-    }
-
-    @Override
-    public String getSharingProfileIdentifier() {
-
-        // Return sharing profile identifier if known
-        if (sharingProfile != null)
-            return sharingProfile.getIdentifier();
-
-        // No associated sharing profile
-        return null;
-
-    }
-
-    @Override
-    public String getSharingProfileName() {
-
-        // Return sharing profile name if known
-        if (sharingProfile != null)
-            return sharingProfile.getName();
-
-        // No associated sharing profile
-        return null;
-
-    }
-
-    @Override
-    public Date getStartDate() {
-        return startDate;
-    }
-
-    @Override
-    public Date getEndDate() {
-
-        // Active connections have not yet ended
-        return null;
-        
-    }
-
-    @Override
-    public String getRemoteHost() {
-        return user.getRemoteHost();
-    }
-
-    @Override
-    public String getUsername() {
-        return user.getIdentifier();
-    }
-
     @Override
     public boolean isActive() {
         return tunnel != null && tunnel.isOpen();
@@ -387,7 +390,7 @@ public class ActiveConnectionRecord implements ConnectionRecord {
             
             @Override
             public UUID getUUID() {
-                return uuid;
+                return ActiveConnectionRecord.this.getUUID();
             }
 
         };
@@ -401,18 +404,6 @@ public class ActiveConnectionRecord implements ConnectionRecord {
         
     }
 
-    /**
-     * Returns the UUID of the underlying tunnel. If there is no underlying
-     * tunnel, this will be the UUID assigned to the underlying tunnel when the
-     * tunnel is set.
-     *
-     * @return
-     *     The current or future UUID of the underlying tunnel.
-     */
-    public UUID getUUID() {
-        return uuid;
-    }
-
     /**
      * Returns the connection ID of the in-progress connection as determined by
      * guacd, not to be confused with the connection identifier determined by
