@@ -19,11 +19,13 @@
 
 package org.apache.guacamole.auth.quickconnect;
 
+import com.google.inject.Inject;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.guacamole.GuacamoleException;
 import org.apache.guacamole.auth.quickconnect.utility.QCParser;
+import org.apache.guacamole.auth.quickconnect.conf.ConfigurationService;
 import org.apache.guacamole.net.auth.ConnectionGroup;
 import org.apache.guacamole.net.auth.simple.SimpleConnection;
 import org.apache.guacamole.net.auth.simple.SimpleDirectory;
@@ -35,40 +37,46 @@ import org.apache.guacamole.protocol.GuacamoleConfiguration;
  * completely in memory.
  */
 public class QuickConnectDirectory extends SimpleDirectory<Connection> {
-
+    
+    /**
+     * The configuration service for the QuickConnect module.
+     */
+    @Inject
+    private ConfigurationService confService;
+    
     /**
      * The connections to store.
      */
     private final Map<String, Connection> connections =
-            new ConcurrentHashMap<String, Connection>();
+            new ConcurrentHashMap<>();
 
     /**
      * The root connection group for this directory.
      */
-    private final QuickConnectionGroup rootGroup;
+    private QuickConnectionGroup rootGroup;
 
     /**
      * The internal counter for connection IDs.
      */
-    private final AtomicInteger connectionId;
+    private AtomicInteger connectionId;
 
     /**
-     * Creates a new QuickConnectDirectory with the default
-     * empty Map for Connection objects, and the specified
-     * ConnectionGroup at the root of the directory.
+     * Initialize the QuickConnectDirectory with the default empty Map for
+     * Connection objects, and the specified ConnectionGroup at the root of
+     * the directory.
      *
      * @param rootGroup
      *     A group that should be at the root of this directory.
      */
-    public QuickConnectDirectory(ConnectionGroup rootGroup) {
+    public void init(ConnectionGroup rootGroup) {
         this.rootGroup = (QuickConnectionGroup)rootGroup;
         this.connectionId = new AtomicInteger();
         super.setObjects(this.connections);
+        
     }
 
     /**
-     * Returns the current connection identifier counter and
-     * then increments it.
+     * Returns the current connection identifier counter and then increments it.
      *
      * @return
      *     An int representing the next available connection
@@ -84,13 +92,14 @@ public class QuickConnectDirectory extends SimpleDirectory<Connection> {
     }
 
     /**
-     * Create a SimpleConnection object from a GuacamoleConfiguration,
-     * obtain an identifier, and place it on the tree, returning the
-     * identifier value of the new connection.
+     * Taking a URI, parse the URI into a GuacamoleConfiguration object and
+     * then use the configuration to create a SimpleConnection object, obtain
+     * an identifier, and place it on the tree, returning the identifier value
+     * of the new connection.
      *
-     * @param config
-     *     The GuacamoleConfiguration object to use to create the
-     *     SimpleConnection object.
+     * @param uri
+     *     The URI to parse into a GuacamoleConfiguration, which will then be
+     *     used to generate the SimpleConnection.
      *
      * @return
      *     The identifier of the connection created in the directory.
@@ -98,13 +107,20 @@ public class QuickConnectDirectory extends SimpleDirectory<Connection> {
      * @throws GuacamoleException
      *     If an error occurs adding the object to the tree.
      */
-    public String create(GuacamoleConfiguration config) throws GuacamoleException {
+    public String create(String uri) throws GuacamoleException {
 
         // Get the next available connection identifier.
         String newConnectionId = Integer.toString(getNextConnectionID());
 
+        // Get a new QCParser
+        QCParser parser = new QCParser(confService.getAllowedParameters(),
+                                        confService.getDeniedParameters());
+        
+        // Parse the URI into a configuration
+        GuacamoleConfiguration config = parser.getConfiguration(uri);
+
         // Generate a name for the configuration.
-        String name = QCParser.getName(config);
+        String name = parser.getName(config);
 
         // Create a new connection and set the parent identifier.
         Connection connection = new SimpleConnection(name, newConnectionId, config, true);
